@@ -417,4 +417,4 @@ class EMailParserType(ReadableEnum):
     ''' 电子邮件 parser 类型'''
 
     E_PARSER_NAV = 1  # 净值邮件
-    E_PARSER_VALUATION = 2  # 估值邮件
+    E_PARSER_VALUATION = 2  # 估值表邮件
