@@ -116,7 +116,7 @@ class PortfolioDataApi(metaclass=Singleton):
                 mutual_fund_info = BasicDataApi().get_fund_info(fund_list=mutual_fund_list)[['fund_id','desc_name']]
                 mutual_fund_info.loc[:,'fund_type'] = '公募'    
             if len(hedge_fund_list) > 0:
-                hedge_fund_info = RawDataApi().get_hf_fund_info(fund_list=hedge_fund_list)[['fund_id','desc_name']]
+                hedge_fund_info = BasicDataApi().get_fof_info_strategy_exsit(fof_id_list=hedge_fund_list)[['fof_id','fof_name']].rename(columns={'fof_id':'fund_id','fof_name':'desc_name'})
                 hedge_fund_info.loc[:,'fund_type'] = '私募'
             df_list = [i for i in [mutual_fund_info, hedge_fund_info] if not i.empty ]
             fund_info = pd.concat(df_list)
@@ -135,7 +135,7 @@ class PortfolioDataApi(metaclass=Singleton):
                 mutual_fund_info = BasicDataApi().get_fund_info(fund_list=mutual_fund_list)[['fund_id','desc_name']]
                 mutual_fund_info.loc[:,'fund_type'] = '公募'    
             if len(hedge_fund_list) > 0:
-                hedge_fund_info = RawDataApi().get_hf_fund_info(fund_list=hedge_fund_list)[['fund_id','desc_name']]
+                hedge_fund_info = BasicDataApi().get_fof_info_strategy_exsit(fof_id_list=hedge_fund_list)[['fof_id','fof_name']].rename(columns={'fof_id':'fund_id','fof_name':'desc_name'})
                 hedge_fund_info.loc[:,'fund_type'] = '私募'
             df_list = [i for i in [mutual_fund_info, hedge_fund_info] if not i.empty ]
             fund_info = pd.concat(df_list)
@@ -161,10 +161,10 @@ class PortfolioDataApi(metaclass=Singleton):
                 if len(mutual_fund_list) > 0:
                     mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=_start_date,end_date=end_date,fund_list=mutual_fund_list)
                 if len(hedge_fund_list) > 0:
-                    hedge_fund_nav = RawDataApi().get_hf_fund_nav(fund_ids=hedge_fund_list, start_date=_start_date, end_date=end_date)
+                    hedge_fund_nav = DerivedDataApi().get_fof_nav_public_adj(fof_id_list=hedge_fund_list, start_date=_start_date, end_date=end_date)
             else:
                 mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=_start_date,end_date=end_date)
-                hedge_fund_nav = RawDataApi().get_hf_fund_nav_dt(start_date=_start_date, end_date=end_date)
+                hedge_fund_nav = DerivedDataApi().get_fof_nav_public_adj_dt(start_date=_start_date, end_date=end_date)
             
             if not mutual_fund_nav.empty:
                 mutual_fund_nav = mutual_fund_nav.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value')
@@ -371,11 +371,12 @@ class PortfolioDataApi(metaclass=Singleton):
             fund_nav = fund_nav.dropna()
             fund_nav.columns=['组合']
             basic = BasicDataApi()
+            derived = DerivedDataApi()
             order_list, fund_type = self.trade_history_parse(trade_history)
             fund_list = set(list([j for dic in order_list for i in dic.values() for j in i.keys()]))
             hf_fund_con = any(['!' not in _fund_id for _fund_id in fund_list])
             frequency = '1W' if hf_fund_con else '1D'
-            index_price = basic.get_index_and_asset_price(index_list=[index_id],begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
+            index_price = self.get_index_and_asset_price(index_list=[index_id],begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
             name_dic = basic.get_index_and_asset_name_dic([index_id])
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
             df = pd.merge(fund_nav.reset_index(), index_price.reset_index(),how='outer').set_index('datetime').sort_index().ffill().dropna().reindex(fund_nav.index).bfill()
@@ -402,8 +403,8 @@ class PortfolioDataApi(metaclass=Singleton):
             sharpe = round(bk_stats['sharpe'],2)
             st1 = f'年化收益: {ar} 年化波动: {av} 夏普比率: {sharpe}'
             st2 = f'最近回撤: {r_mdd} 最近回撤开始 {r_mdd_b1}'
-            st3 = f'最大回撤 {mdd1} {mdd1_d1} : {mdd1_d2}, {mdd1_lens} days'
-            st4 = f'第二回撤 {mdd2} {mdd2_d1} : {mdd2_d2}, {mdd2_lens} days'
+            st3 = f'最大回撤恢复 {mdd1} {mdd1_d1} : {mdd1_d2}, {mdd1_lens} days'
+            st4 = f'第二回撤恢复 {mdd2} {mdd2_d1} : {mdd2_d2}, {mdd2_lens} days'
             
             mdd_dic_1 = {
                 'd1':str(mdd_details['mdd1_d1']),
@@ -513,7 +514,8 @@ class PortfolioDataApi(metaclass=Singleton):
             hf_fund_con = any(['!' not in _fund_id for _fund_id in fund_list])
             frequency = '1W' if hf_fund_con else '1D'
             basic = BasicDataApi()
-            index_price = basic.get_index_and_asset_price(index_list=[index_id],begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
+            derived = DerivedDataApi()
+            index_price = self.get_index_and_asset_price(index_list=[index_id],begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
             name_dic = basic.get_index_and_asset_name_dic([index_id])
             df = pd.merge(fund_nav.reset_index(), index_price.reset_index(),how='outer').set_index('datetime').sort_index().ffill().dropna().reindex(fund_nav.index)
             df = df / df.iloc[0]
@@ -652,6 +654,7 @@ class PortfolioDataApi(metaclass=Singleton):
             fund_type_dic = { vi: benchmark_id_info[k] for k, v in fund_type.items() for vi in v}
             index_id = fund_type_dic[fund_id]
             basic = BasicDataApi()
+            derived = DerivedDataApi()
             fund_list = [fund_id]
             fund_info = self.get_part_fund_info(fund_list=fund_list)
             desc_name_dic = fund_info.set_index('fund_id')['desc_name'].to_dict()
@@ -669,7 +672,7 @@ class PortfolioDataApi(metaclass=Singleton):
                 }
                 return data
             fund_nav = fund_nav.ffill()
-            index_price = basic.get_index_and_asset_price(index_list=[index_id],begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
+            index_price = self.get_index_and_asset_price(index_list=[index_id],begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
             nav_df = fund_nav.join(index_price).ffill().dropna()
             
             result = []
@@ -908,6 +911,7 @@ class PortfolioDataApi(metaclass=Singleton):
 
     def get_port_fund_alpha_detail(self, trade_history, benchmark_id_info): 
         try:
+            derived = DerivedDataApi()
             order_list, fund_type = self.trade_history_parse(trade_history)
             result = PortfolioDataApi().get_portfolio_nav(trade_history=trade_history)
             fund_nav = result['各基净值']
@@ -918,7 +922,7 @@ class PortfolioDataApi(metaclass=Singleton):
             dts_list = [j for i in order_list for j in i.keys()] + [fund_nav.index[-1]]
             dts_list = [[i,dts_list[dts_list.index(i)+1]] for i in dts_list if i != dts_list[-1]]
             basic = BasicDataApi()
-            index_price = basic.get_index_and_asset_price(index_list=list(benchmark_id_info.values()),begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
+            index_price = self.get_index_and_asset_price(index_list=list(benchmark_id_info.values()),begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
             fund_info = self.get_all_fund_info(fund_list=fund_nav.columns.tolist())
             fund_desc_dic = fund_info.set_index('fund_id')['desc_name'].to_dict()
             index_desc_dic = basic.get_index_and_asset_name_dic(list(benchmark_id_info.values()))
@@ -1002,7 +1006,7 @@ class PortfolioDataApi(metaclass=Singleton):
 
             _index_price = result['大类净值'].loc[begin_date:]
             benchmark_id_info_r = {v: k for k, v in benchmark_id_info.items()}
-            index_price = BasicDataApi().get_index_and_asset_price(index_list=list(benchmark_id_info.values()),begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
+            index_price = self.get_index_and_asset_price(index_list=list(benchmark_id_info.values()),begin_date=fund_nav.index[0], end_date=fund_nav.index[-1])
             index_price = index_price.rename(columns=benchmark_id_info_r)
             index_price = index_price.reindex(_index_price.index.union(index_price.index)).ffill().bfill().reindex(_index_price.index)
             result_2 = _index_price / _index_price.iloc[0] / (index_price / index_price.iloc[0])
@@ -1171,6 +1175,456 @@ class PortfolioDataApi(metaclass=Singleton):
                     '最近三年收益率':None,
                     '最近五年收益率':None,
                 }
-            
 
+    def get_asset_price(self, begin_date, end_date, time_para, asset_list: Tuple[str] = ()):
+        try:
+            begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
+            raw = RawDataApi()
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            asset_info = basic.get_asset_info(asset_id_list=asset_list)
+            asset_list = asset_info.asset_id.tolist()
+            _asset_list = [i for i in asset_list if i.split('-')[0] == 'fund']
+            _res = []
+            for r in asset_info.itertuples():
+                if r.asset_id in _asset_list:
+                    _res.append(r.real_name)
+                else:
+                    _res.append(r.asset_name)
+            asset_info.loc[:,'show_name'] = _res
+            inputs_asset_list = asset_info.asset_id.tolist()
+            asset_dic = asset_info.set_index('asset_id').to_dict()['real_id']
+            asset_list_0 = ['asset-dollar']
+            asset_list_1 = ['asset-btc']
+            asset_list_2 = ['asset-usdebt']
+            _asset_list_0 = [i for i in inputs_asset_list if i in asset_list_0]
+            _asset_list_1 = [i for i in inputs_asset_list if i in asset_list_1]
+            _asset_list_2 = [i for i in inputs_asset_list if i in asset_list_2]
+            _asset_list_3 = [i for i in inputs_asset_list if i.split('-')[0] == 'fund']
+            _asset_list_4 = [i for i in inputs_asset_list if i.split('-')[0] == 'stgindex']
+            _asset_list_41 = [i for i in inputs_asset_list if i.split('-')[0] == 'comedy']
+            _asset_sum = _asset_list_0 + _asset_list_1 + _asset_list_2 + _asset_list_3 + _asset_list_4 + _asset_list_41
+            _asset_list_5 = [i for i in inputs_asset_list if i not in _asset_sum]
+            _asset_list_6 = [i for i in inputs_asset_list if i.split('-')[0] == 'macro']
 
+            result = []
+            if len(_asset_list_0+_asset_list_6) > 0:
+                _asset_list_0 = _asset_list_0+_asset_list_6
+                real_ids = [asset_dic[i] for i in _asset_list_0]
+                _df = raw.get_em_macroeconomic_daily(codes=real_ids,start_date=begin_date,end_date=end_date).pivot_table(index='datetime',columns='codes',values='value')
+                result.append(_df)
+            if len(_asset_list_1) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_1]
+                _df = raw.get_btc(asset_ids=real_ids,start_date=begin_date,end_date=end_date).pivot_table(index='date',columns='codes',values='close')
+                result.append(_df)
+            if len(_asset_list_2) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_2]
+                _df = raw.get_em_future_price(future_ids=real_ids,start_date=begin_date,end_date=end_date).pivot_table(index='datetime',columns='future_id',values='close')
+                result.append(_df)
+            if len(_asset_list_3) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_3]
+                _df = derived.get_fof_nav_public_adj(fund_ids=real_ids,start_date=begin_date,end_date=end_date).pivot_table(index='datetime',columns='fund_id',values='nav')
+                result.append(_df)
+            if len(_asset_list_4) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_4]
+                _df = raw.get_hf_index_price(index_ids=real_ids,start_date=begin_date,end_date=end_date).pivot_table(index='index_date',columns='index_id',values='close')
+                result.append(_df)
+            if len(_asset_list_41) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_41]
+                _df = raw.get_em_future_price(future_ids=real_ids,start_date=begin_date,end_date=end_date).pivot_table(index='datetime',columns='future_id',values='close')
+                result.append(_df)
+            if len(_asset_list_5) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_5]
+                _df = basic.get_index_price_dt(index_list=real_ids,start_date=begin_date,end_date=end_date).pivot_table(index='datetime',columns='index_id',values='close')
+                result.append(_df)
+
+            df_result = pd.concat(result,axis=1).sort_index()
+            df_result = df_result / df_result.bfill().iloc[0]
+            df_result_show = df_result.rename(columns=asset_info.set_index('real_id').to_dict()['show_name'])
+            df_result_real = df_result.ffill().rename(columns=asset_info.set_index('real_id').to_dict()['real_name'])
+            df_result_show.index.name = 'datetime'
+            df_result_real.index.name = 'datetime'
+            asset_info = asset_info.drop_duplicates(subset=['real_id'])
+            dic = {'data':df_result_show,'_input_asset_nav':df_result_real,'_input_asset_info':asset_info}
+            return dic
+        
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.get_asset_price')
+
+    def get_asset_price_dt(self, asset_list, begin_date, end_date):
+        try:
+            raw = RawDataApi()
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            _begin_date = begin_date
+            #asset_list = ['asset-big_cap','stgindex-bond','theme-csi_const_eng','industry-mech']
+            asset_info = basic.get_asset_info(asset_id_list=asset_list)
+            asset_dic = asset_info.set_index('asset_id').to_dict()['real_id']
+            asset_info = basic.get_asset_info(asset_id_list=asset_list)
+            inputs_asset_list = asset_list
+            asset_list_0 = ['asset-dollar']
+            asset_list_1 = ['asset-btc']
+            asset_list_2 = ['asset-usdebt']
+            _asset_list_0 = [i for i in inputs_asset_list if i in asset_list_0]
+            _asset_list_1 = [i for i in inputs_asset_list if i in asset_list_1]
+            _asset_list_2 = [i for i in inputs_asset_list if i in asset_list_2]
+            _asset_list_3 = [i for i in inputs_asset_list if i.split('-')[0] == 'fund']
+            _asset_list_4 = [i for i in inputs_asset_list if i.split('-')[0] == 'stgindex']
+            _asset_list_41 = [i for i in inputs_asset_list if i.split('-')[0] == 'comedy']
+            _asset_sum = _asset_list_0 + _asset_list_1 + _asset_list_2 + _asset_list_3 + _asset_list_4 + _asset_list_41
+            _asset_list_5 = [i for i in inputs_asset_list if i not in _asset_sum]
+
+            result = []
+            if len(_asset_list_0) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_0]
+                _df = raw.get_em_macroeconomic_daily(codes=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='codes',values='value')
+                result.append(_df)
+            if len(_asset_list_1) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_1]
+                _df = raw.get_btc(asset_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='date',columns='codes',values='close')
+                result.append(_df)
+            if len(_asset_list_2) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_2]
+                _df = raw.get_em_future_price(future_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='future_id',values='close')
+                result.append(_df)
+            if len(_asset_list_3) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_3]
+                _df = derived.get_fof_nav_public_adj(fund_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='fund_id',values='nav')
+                result.append(_df)
+            if len(_asset_list_4) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_4]
+                _df = raw.get_hf_index_price(index_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='index_date',columns='index_id',values='close')
+                result.append(_df)
+            if len(_asset_list_41) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_41]
+                _df = raw.get_em_future_price(future_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='future_id',values='close')
+                result.append(_df)
+            if len(_asset_list_5) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_5]
+                _df = basic.get_index_price_dt(index_list=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='index_id',values='close')
+                result.append(_df)
+
+            df_result = pd.concat(result,axis=1).sort_index()
+            df_result = df_result.bfill().ffill()
+            return df_result
+
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.get_asset_price_dt')
+            return pd.DataFrame()
+
+    def get_asset_price_dt_back_with_asset_id(self, asset_list, begin_date, end_date):
+        try:
+            raw = RawDataApi()
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            _begin_date = begin_date
+            #asset_list = ['asset-big_cap','stgindex-bond','theme-csi_const_eng','industry-mech']
+            asset_info = basic.get_asset_info(asset_id_list=asset_list)
+            asset_dic = asset_info.set_index('asset_id').to_dict()['real_id']
+            asset_info = basic.get_asset_info(asset_id_list=asset_list)
+            asset_asset_id_dic = asset_info.set_index('real_id')['asset_id'].to_dict()
+            inputs_asset_list = asset_list
+            asset_list_0 = ['asset-dollar']
+            asset_list_1 = ['asset-btc']
+            asset_list_2 = ['asset-usdebt']
+            _asset_list_0 = [i for i in inputs_asset_list if i in asset_list_0]
+            _asset_list_1 = [i for i in inputs_asset_list if i in asset_list_1]
+            _asset_list_2 = [i for i in inputs_asset_list if i in asset_list_2]
+            _asset_list_3 = [i for i in inputs_asset_list if i.split('-')[0] == 'fund']
+            _asset_list_4 = [i for i in inputs_asset_list if i.split('-')[0] == 'stgindex']
+            _asset_list_41 = [i for i in inputs_asset_list if i.split('-')[0] == 'comedy']
+            _asset_sum = _asset_list_0 + _asset_list_1 + _asset_list_2 + _asset_list_3 + _asset_list_4 + _asset_list_41
+            _asset_list_5 = [i for i in inputs_asset_list if i not in _asset_sum]
+
+            result = []
+            if len(_asset_list_0) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_0]
+                _df = raw.get_em_macroeconomic_daily(codes=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='codes',values='value')
+                result.append(_df)
+            if len(_asset_list_1) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_1]
+                _df = raw.get_btc(asset_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='date',columns='codes',values='close')
+                result.append(_df)
+            if len(_asset_list_2) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_2]
+                _df = raw.get_em_future_price(future_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='future_id',values='close')
+                result.append(_df)
+            if len(_asset_list_3) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_3]
+                _df = derived.get_fof_nav_public_adj(fund_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='fund_id',values='nav')
+                result.append(_df)
+            if len(_asset_list_4) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_4]
+                _df = raw.get_hf_index_price(index_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='index_date',columns='index_id',values='close')
+                result.append(_df)
+            if len(_asset_list_41) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_41]
+                _df = raw.get_em_future_price(future_ids=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='future_id',values='close')
+                result.append(_df)
+            if len(_asset_list_5) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_5]
+                _df = basic.get_index_price_dt(index_list=real_ids,start_date=_begin_date,end_date=end_date).pivot_table(index='datetime',columns='index_id',values='close')
+                result.append(_df)
+
+            df_result = pd.concat(result,axis=1).sort_index()
+            df_result = df_result.bfill().ffill()
+            df_result = df_result.rename(columns=asset_asset_id_dic)
+            return df_result
+
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.get_asset_price_dt')
+            return pd.DataFrame()
+
+    def get_product_price(self, begin_date, end_date, time_para, product_list: Tuple[str] = (), price_type:str='价格线', benchmark_id:str='hs300'):#'价格比线'
+        try:
+            begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
+            raw = RawDataApi()
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            hf_info = basic.get_fof_info_strategy_exsit(fof_id_list=product_list)
+            name_dic = hf_info.set_index('fof_id')['fof_name'].to_dict()
+            asset_info = basic.get_asset_info_real(real_id_list=product_list)
+            asset_info = asset_info.drop_duplicates('real_id')
+
+            if not asset_info.empty:
+                _name_dic = asset_info.set_index('real_id').to_dict()['real_name']
+                for k, v in _name_dic.items():
+                    name_dic[k] = v
+            _df = hf_info.rename(columns={'fof_id':'real_id','strategy_type':'asset_name','fof_name':'real_name'})[['real_id','asset_name','real_name']]
+            if not _df.empty:
+                _df.loc[:,'asset_type'] = '典型产品'
+            asset_info = asset_info.append(_df).drop_duplicates('real_id')
+            result = []
+            real_ids = product_list
+            _df = derived.get_fof_nav_public_adj(fund_ids=real_ids,start_date=begin_date,end_date=end_date)
+            if not _df.empty > 0:
+                _df = _df.pivot_table(index='datetime',columns='fund_id',values='nav')
+                result.append(_df)
+            _df = basic.get_index_price_dt(index_list=real_ids,start_date=begin_date,end_date=end_date)
+            if not _df.empty > 0:
+                _df = _df.pivot_table(index='datetime',columns='index_id',values='close')
+                result.append(_df)
+            df_result = pd.concat(result,axis=1).sort_index()
+            if price_type == '价格线':
+                pass
+            elif price_type == '价格比线':
+                index_price = basic.get_index_price_dt(index_list=[benchmark_id],start_date=begin_date,end_date=end_date)
+                index_price = index_price.pivot_table(index='datetime', values='close', columns='index_id')
+                index_price = index_price.reindex(df_result.index.union(index_price.index)).sort_index().ffill().reindex(df_result.index)
+                index_price.columns = ['benchmark']
+                result = []
+                for fund_id in df_result:
+                    _df_i = df_result[[fund_id]].dropna()
+                    _df_i = _df_i.join(index_price)
+                    _df_i = _df_i / _df_i.iloc[0]
+                    _df_i[fund_id] = _df_i[fund_id] / _df_i.benchmark
+                    result.append(_df_i[[fund_id]])
+                df_result = pd.concat(result, axis=1).sort_index()
+            df_result = df_result / df_result.bfill().iloc[0]
+            df_result = df_result.rename(columns=name_dic)
+            dic = {'data':df_result,'_input_asset_nav':df_result.ffill(),'_input_asset_info':asset_info}
+            return dic
+        
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.get_product_price')
+
+    def product_recent_rate(self, product_list, year:int=None, weekly:bool=False):
+        try:
+            raw = RawDataApi()
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            hf_info = basic.get_fof_info_strategy_exsit()
+            name_dic = hf_info.set_index('fof_id')['fof_name'].to_dict()
+            asset_info = basic.get_asset_info_real(real_id_list=product_list)
+            asset_info = asset_info.drop_duplicates('real_id')
+            if not asset_info.empty:
+                _name_dic = asset_info.set_index('real_id').to_dict()['real_name']
+                for k, v in _name_dic.items():
+                    name_dic[k] = v
+            result = []
+            real_ids = product_list
+            _df = derived.get_fof_nav_public_adj(fund_ids=real_ids,start_date='20091231')
+            if not _df.empty > 0:
+                _df = _df.pivot_table(index='datetime',columns='fund_id',values='nav')
+                result.append(_df)
+            _df = basic.get_index_price_dt(start_date='20091231',index_list=real_ids)
+            if not _df.empty > 0:
+                _df = _df.pivot_table(index='datetime',columns='index_id',values='close')
+                result.append(_df)
+
+            index_price = pd.concat(result,axis=1).sort_index().ffill().dropna()
+            if weekly:
+                # 周度
+                index_price_w = basic.data_resample_monthly_nav(index_price,rule='1W').bfill()
+                index_ret = index_price_w.pct_change(1).dropna().tail(12)
+                td = index_ret.index
+                td = [i.strftime('%Y%m%d') for i in td]
+            else:
+                if year is None:
+                    # 年度
+                    index_price_y = basic.data_resample_monthly_nav(index_price.bfill(),rule='12M')
+                    index_ret_year = index_price_y.pct_change(1).dropna()
+                    td = index_ret_year.index
+                    td = [str(i.year) for i in td]
+                    index_ret = index_ret_year.copy()
+                else:
+                    index_price_m = index_price.loc[datetime.date(year-1,12,31):datetime.date(year,12,31)]
+                    index_price_m = basic.data_resample_monthly_nav(index_price_m,rule='1M').bfill()
+                    index_ret_m = index_price_m.pct_change(1).dropna()
+                    td = index_ret_m.index
+                    td = [str(i.year) + str(i.month).zfill(2) for i in td ]
+                    index_ret = index_ret_m.copy()
+            index_ret.index = td
+            index_ret.loc['均值',:] = index_ret.mean()
+            index_ret = index_ret.round(4)*100
+            index_ret = index_ret.rename(columns=name_dic)
+            index_ret = index_ret.T
+            index_ret.index.name = 'index_id'
+            return index_ret
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.product_recent_rate')
+
+    def asset_recent_rate(self, asset_list, year:int=None, weekly:bool=False):
+        #如果year为空 返回年度收益
+        #如果year为整数 返回当年月度收益 year从2010开始
+        try:
+            raw = RawDataApi()
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            asset_info = basic.get_asset_info(asset_id_list=asset_list)
+            asset_list = asset_info.asset_id.tolist()
+            _asset_list = [i for i in asset_list if i.split('-')[0] == 'fund']
+            _res = []
+            for r in asset_info.itertuples():
+                if r.asset_id in _asset_list:
+                    _res.append(r.real_name)
+                else:
+                    _res.append(r.asset_name)
+            asset_info.loc[:,'show_name'] = _res
+            inputs_asset_list = asset_info.asset_id.tolist()
+            asset_dic = asset_info.set_index('asset_id').to_dict()['real_id']
+            name_dic = asset_info.set_index('real_id').to_dict()['real_name']
+            name_dic = {k : v.replace('CFCI','指数')for k, v in name_dic.items()}
+            asset_list_0 = ['asset-dollar']
+            asset_list_1 = ['asset-btc']
+            asset_list_2 = ['asset-usdebt']
+            _asset_list_0 = [i for i in inputs_asset_list if i in asset_list_0]
+            _asset_list_1 = [i for i in inputs_asset_list if i in asset_list_1]
+            _asset_list_2 = [i for i in inputs_asset_list if i in asset_list_2]
+            _asset_list_3 = [i for i in inputs_asset_list if i.split('-')[0] == 'fund']
+            _asset_list_4 = [i for i in inputs_asset_list if i.split('-')[0] == 'stgindex']
+            _asset_sum = _asset_list_0 + _asset_list_1 + _asset_list_2 + _asset_list_3 + _asset_list_4
+            _asset_list_5 = [i for i in inputs_asset_list if i not in _asset_sum]
+
+            result = []
+            if len(_asset_list_0) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_0]
+                _df = raw.get_em_macroeconomic_daily(codes=real_ids,start_date='20091231').pivot_table(index='datetime',columns='codes',values='value')
+                result.append(_df)
+            if len(_asset_list_1) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_1]
+                _df = raw.get_btc(asset_ids=real_ids,start_date='20091231').pivot_table(index='date',columns='codes',values='close')
+                result.append(_df)
+            if len(_asset_list_2) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_2]
+                _df = raw.get_em_future_price(future_ids=real_ids,start_date='20091231').pivot_table(index='datetime',columns='future_id',values='close')
+                result.append(_df)
+            if len(_asset_list_3) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_3]
+                _df = derived.get_fof_nav_public_adj(fund_ids=real_ids,start_date='20091231').pivot_table(index='datetime',columns='fund_id',values='nav')
+                result.append(_df)
+            if len(_asset_list_4) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_4]
+                _df = raw.get_hf_index_price(index_ids=real_ids,start_date='20091231').pivot_table(index='index_date',columns='index_id',values='close')
+                result.append(_df)
+            if len(_asset_list_5) > 0:
+                real_ids = [asset_dic[i] for i in _asset_list_5]
+                _df = basic.get_index_price_dt(index_list=real_ids,start_date='20091231').pivot_table(index='datetime',columns='index_id',values='close')
+                result.append(_df)
+
+            index_price = pd.concat(result,axis=1).sort_index().ffill().dropna().rename(columns=name_dic)
+            if weekly:
+                index_price_w = basic.data_resample_monthly_nav(index_price,rule='1W').bfill()
+                index_ret = index_price_w.pct_change(1).dropna().tail(12)
+                td = index_ret.index
+                td = [i.strftime('%Y%m%d') for i in td]
+            else:
+                if year is None:
+                    # 年度
+                    ## 年度收益净值调整
+                    b_d = index_price.index.values[0]
+                    if b_d < datetime.date(b_d.year,12,1):
+                        _b_d = datetime.date(b_d.year-1,12,31)
+                        index_price.loc[_b_d] = index_price.iloc[0]
+                        index_price = index_price.sort_index()
+                    else:
+                        _b_d = index_price.loc[:datetime.date(b_d.year,12,31)].index[0]
+                        index_price = index_price.loc[_b_d:].copy()
+                    index_price_y = basic.data_resample_monthly_nav(index_price.bfill(),rule='12M')
+                    index_ret_year = index_price_y.pct_change(1).dropna()
+                    td = index_ret_year.index
+                    td = [str(i.year) for i in td]
+                    index_ret = index_ret_year.copy()
+                else:
+                    index_price_m = index_price.loc[datetime.date(year-1,12,31):datetime.date(year,12,31)]
+                    index_price_m = basic.data_resample_monthly_nav(index_price_m,rule='1M').bfill()
+                    index_ret_m = index_price_m.pct_change(1).dropna()
+                    td = index_ret_m.index
+                    td = [str(i.year) + str(i.month).zfill(2) for i in td ]
+                    index_ret = index_ret_m.copy()
+            last_day = td[-1]
+            index_ret.index = td
+            index_ret = index_ret.T.sort_values(last_day, ascending=False).T
+            index_ret.loc['均值',:] = index_ret.mean()
+            index_ret = index_ret.round(4)*100
+            index_ret = index_ret.T
+            index_ret.index.name = 'index_id'
+            return index_ret
+
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.asset_recent_rate')
+
+    def get_all_asset_price(self, start_date:str='', end_date:str='', id_list:list=[]):
+        try:
+            result = []
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            index_price = basic.get_index_price_dt(start_date=begin_date, end_date=end_date, index_list=id_list)
+            if not index_price.empty:
+                index_price = index_price.pivot_table(index='datetime', values='close', columns='index_id')
+                result.append(index_price)
+            fund_nav = basic.get_fund_nav_with_date(start_date=begin_date, end_date=end_date, fund_list=id_list)
+            if not fund_nav.empty:
+                fund_nav = fund_nav.pivot_table(index='datetime', values='adjusted_net_value', columns='fund_id')
+                result.append(fund_nav)
+            hf_fund_nav = derived.get_fof_nav_public_adj(fund_ids=id_list, start_date=begin_date, end_date=end_date)
+            if not hf_fund_nav.empty:
+                hf_fund_nav = hf_fund_nav.pivot_table(index='datetime', values='nav', columns='fund_id')
+                result.append(hf_fund_nav)
+            fund_nav = pd.concat(result, axis=1).sort_index().dropna()
+            return fund_nav
+
+        except Exception as e:
+            print(f'Failed to get data <err_msg> {e} from port.get_all_asset_price')
+            return []
+
+    def get_index_and_asset_price(self, index_list, begin_date, end_date):
+        try:
+            basic = BasicDataApi()
+            derived = DerivedDataApi()
+            index_id_list = [i for i in index_list if '-' not in i]
+            asset_id_list = [i for i in index_list if '-' in i]
+            index_price = pd.DataFrame()
+            asset_price = pd.DataFrame()
+            if len(index_id_list) > 0:
+                #print(f'index_ids {index_id_list}')
+                index_price = basic.get_index_price_dt(index_list=index_id_list, start_date=begin_date, end_date=end_date)
+                index_price = index_price.pivot_table(index='datetime',columns='index_id',values='close')
+            if len(asset_id_list) > 0:
+                #print(f'asset_id_list {asset_id_list}')
+                asset_price = derived.get_asset_price_dt_back_with_asset_id(asset_list=asset_id_list,begin_date=begin_date-datetime.timedelta(days=10),end_date=end_date)
+            df = pd.concat([index_price,asset_price],join='outer', axis=1).sort_index().ffill().reindex(index_price.index)
+            return df
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.get_index_and_asset_price')
+            return pd.DataFrame()
\ No newline at end of file
