@@ -578,6 +578,14 @@ class FundDataManager(DataManager):
             self._score_manager.pre_calculate(is_filter_c=True, use_weekly_monthly_indicators=use_weekly_monthly_indicators)
             self._score_manager.pre_calculate_manager()
 
+        failed_tasks = self._s3_retriever.retrieve_dts(self.dts, ['mng_indicator_score', 'fund_indicator_score'])
+        if failed_tasks:
+            print(f'retrieve dm failed from s3 (err_info){failed_tasks}')
+
+        failed_tasks = self._s3_retriever.retrieve_misc(self, ['mng_index_list'])
+        if failed_tasks:
+            print(f'retrieve dm failed from s3 (err_info){failed_tasks}')
+
         # 干掉不用的字段以节省内存
         self.dts.remove_fields(block_df_list)
 
