@@ -25,7 +25,9 @@ class HedgeFundDataManager(metaclass=Singleton):
     def __init__(self):
         pass
 
-    def _get_total_share(self, manager_id: str, fof_id: str, date: datetime.date, investor_id: str = '', exclude1: List[int] = [], exclude2: List[str] = []) -> Tuple[float, pd.DataFrame]:
+    def _get_total_share(self, manager_id: str, fof_id: str, date: str, investor_id: str = '', exclude1: List[int] = [], exclude2: List[str] = []) -> Tuple[float, pd.DataFrame]:
+        date = pd.to_datetime(date, infer_datetime_format=True).date()
+
         part1 = DerivedDataApi().get_hedge_fund_investor_pur_redemp(manager_id, [fof_id])
         assert part1 is not None, '!!!!'
         part1 = part1[part1.datetime <= date]
