@@ -72,8 +72,6 @@ class ScoreFunc:
     RetAbilityFundScore: float = 0
     RiskAbilityFundScore: float = 0
     StableAbility: float = 0
-    SelectTimeAbility: float = 0
-    SelectStockAbility: float = 0
     WinRateMonthlyTop50: float = 0
     WinRateMonthlyTop75: float = 0
      #默认条件不选择主动型，对主动性加分数惩罚
@@ -127,8 +125,6 @@ class ScoreFunc:
                 + data.RetAbilityFundScore.mul(other=self.RetAbilityFundScore, fill_value=0) \
                 + data.RiskAbilityFundScore.mul(other=self.RiskAbilityFundScore, fill_value=0) \
                 + data.StableAbility.mul(other=self.StableAbility, fill_value=0) \
-                + data.SelectTimeAbility.mul(other=self.SelectTimeAbility, fill_value=0) \
-                + data.SelectStockAbility.mul(other=self.SelectStockAbility, fill_value=0) \
                 + data.WinRateMonthlyTop50.mul(other=self.WinRateMonthlyTop50, fill_value=0) \
                 + data.WinRateMonthlyTop75.mul(other=self.WinRateMonthlyTop75, fill_value=0) 
                 # + self.down_risk * data.down_risk \
@@ -198,11 +194,13 @@ class FundScoreManager:
                 'default': ScoreFunc(track_err=-0.4,beta=-0.6),
                 'enhanced':ScoreFunc(alpha=0.6, beta=-0.3),
                 'active': ScoreFunc(mng_trade_year=0.1, mng_sharpe=0.3,alpha=0.4,mng_year_ret_std=-0.2,active=10),
+                'active_2': ScoreFunc(WinRateMonthlyTop75=0.5, RiskAbilityFundScore=-0.5,active=10),
             },
             'csi500': {
                 'default': ScoreFunc(info_ratio=0.2,track_err=-0.7, fee_rate=-0.1),
                 'enhanced':ScoreFunc(alpha=0.7, beta=-0.3),
                 'active': ScoreFunc(mng_trade_year=0.1, mng_sharpe=0.3,alpha=0.4,mng_year_ret_std=-0.2,active=10),
+                'active_2': ScoreFunc(WinRateMonthlyTop75=0.5, RiskAbilityFundScore=-0.5,active=10),
             }, 
             'gem': {
                 'default': ScoreFunc(info_ratio=0.2,track_err=-0.7, fee_rate=-0.1),
@@ -513,3 +511,64 @@ class FundScoreManager:
         score_series = cur_d.apply(f, axis=1)
         score_series = dm._score_manager._filter_score_prefer_a(score_series)
         return score_series
+
+class FundRankScoreManager(FundScoreManager):
+
+    def __init__(self, score_filter=ScoreFilter(), score_penalty_param=ScorePenaltyParams()):
+        super().__init__(score_filter=score_filter, score_penalty_param=score_penalty_param)
+        print('#### score rank manager init')
+
+    def _get_score(self, index_id, dt, cur_d, is_filter_c, use_weekly_monthly_indicators=False, score_func: Optional[Union[ScoreFunc, str]] = None):
+        fund_ids = self._filter_size_and_com_hold(dt, index_id, cur_d.index)
+        if fund_ids is not cur_d.index:
+            # 这里没有dropna总计可以快24秒
+            cur_d = cur_d.reindex(fund_ids)
+        func_dic = self.funcs.get(index_id) if score_func is None else {'temp': score_func}
+        if cur_d.shape[0] > 1:
+            #修改因子预处理方式 改用rank
+            #cur_d_sta = cur_d.apply(lambda x: (x - x.mean() + 1e-6)/ (x.std() + 1e-6), axis=0)
+            cur_d_sta = cur_d.rank(ascending=False,pct=True)
+            cur_d_sta['beta'] = cur_d['beta']
+            cur_d_sta['year_length'] = cur_d['year_length']
+            cur_d_sta['active'] = cur_d['active']
+            cur_d_sta['mng_fund_size'] = cur_d['mng_fund_size']
+            less_mng_year = cur_d[cur_d['mng_trade_year'] < 1].index.tolist()
+            cur_d = cur_d_sta
+            score_raw_sum = {}
+            score_sum = {}
+            for type_i, func in func_dic.items():
+                if isinstance(func, str):
+                    try:
+                        FundScoreManager.test_func(func)
+                    except Exception as e:
+                        # func str不合法，直接返回空
+                        print(f'eval fund score func str failed for {index_id} (err_msg){e}')
+                        return {}, {}
+                    score_raw = cur_d.apply(FundScoreManager.get_func(func), axis=1)
+                else:
+                    # 默认均走这个分支
+                    if use_weekly_monthly_indicators:
+                        score_raw = func.get_all(cur_d)
+                    else:
+                        score_raw = func.get(cur_d)
+                if index_id in self.score_penalty_param.FilterYearIndex:
+                    punish_funds = cur_d[cur_d.year_length < self.score_penalty_param.JudgeYearLength].index.tolist()
+                    for _fund_id in punish_funds:
+                        score_raw[_fund_id] += self.score_penalty_param.Penalty
+                    if index_id in ['hs300','csi500'] and type_i in ['active']:
+                        punish_funds_1 = cur_d_sta[cur_d_sta['mng_fund_size'] < 5e8].index.tolist()
+                        punish_funds_1 = cur_d_sta[cur_d_sta['mng_fund_size'] < 5e8].index.tolist()
+                        # 对于管理规模不到5千万和 年限不到1年到经理，进行惩罚
+                        for _fund_id in punish_funds_1:
+                            score_raw[_fund_id] += self.score_penalty_param.Penalty
+                        for _fund_id in less_mng_year:
+                            score_raw[_fund_id] += self.score_penalty_param.Penalty
+                score = (score_raw - score_raw.min()) / (score_raw.max() - score_raw.min())
+                if is_filter_c:
+                    score = self._filter_score(score)
+                score_sum[type_i] = score.to_dict()
+                score_raw_sum[type_i] = score_raw.to_dict()
+        else:
+            score_raw_sum = {type_i : {cur_d.index[0]: 1} for type_i in func_dic}
+            score_sum = score_raw_sum
+        return score_sum, score_raw_sum
\ No newline at end of file
