@@ -579,4 +579,12 @@ class FundPainter(object):
         plt.suptitle(st,y=0.87,fontsize=17)
         plt.grid()
         plt.show()
-        
\ No newline at end of file
+        
+    @staticmethod
+    def plot_legend_aside(df):
+        df.index = [str(i) for i in df.index]
+        df.plot(figsize=(14,8),fontsize=20)
+        plt.title('同期跑赢指数基金',fontsize=25)
+        plt.legend(loc='upper left',fontsize=15, bbox_to_anchor=(1,1))
+        plt.grid()
+        plt.show()
