@@ -64,6 +64,10 @@ class SeriesStatUnit:
     mdd_recover_date2: datetime.date    # 最大回测恢复结束时间
     mdd_recover_lens: int               # 最大回测哈恢复天数
     win_rate_0_period: float            # 设定周期对零胜率
+    nav_density: float                  # 净值密度
+    nav_type: str                       # 净值类型
+    w1_end_date: datetime.date          # 近一周结束时间
+    w1_begin_date: datetime.date        # 近一周开始时间
 
 @dataclasses.dataclass
 class BenchmarkSeiesStatUnit(SeriesStatUnit):
@@ -197,6 +201,10 @@ class Calculator:
             mdd_recover_date2=res['mdd_recover_date2'],
             mdd_recover_lens=res['mdd_recover_lens'],
             win_rate_0_period=res['win_rate_0_period'],
+            nav_density=res['nav_density'],
+            nav_type=res['nav_type'],
+            w1_begin_date=res['w1_begin_date'],
+            w1_end_date=res['w1_end_date'],
         )
 
     @staticmethod
@@ -294,6 +302,10 @@ class Calculator:
             mdd_recover_date2=res['mdd_recover_date2'],
             mdd_recover_lens=res['mdd_recover_lens'],
             win_rate_0_period=res['win_rate_0_period'],
+            nav_density=res['nav_density'],
+            nav_type=res['nav_type'],
+            w1_begin_date=res['w1_begin_date'],
+            w1_end_date=res['w1_end_date'],
         )
 
     @staticmethod
@@ -1060,6 +1072,7 @@ class Calculator:
             _dic = {
                 'pb_mrq':'PB',
                 'pe_ttm_nn':'PE',
+                'pe_pct_5_3':'PE百分位',
                 'roe':'ROE',
                 'ps_ttm':'PS',
                 'dy':'股息率',
@@ -1096,6 +1109,7 @@ class Calculator:
                     df[col] = df[col].round(2)
             df['夏普比率'] = df['夏普比率'].round(2)
             df['卡玛比率'] = df['卡玛比率'].round(2)
+            df.loc[:,'EPS增速'] = round(df.PE / df['预测PEG']/100,2)
             return df.reset_index()
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from get_index_stats.__tablename__')
